/*
 * Copyright © 2024 Igalia S.L.
 * SPDX-License-Identifier: MIT
 */

#pragma once

#include "util/macros.h"

#ifdef __cplusplus
extern "C" {
#endif

enum PACKED isa_cond {
   ISA_COND_TRUE = 0,
   ISA_COND_GT = 1,
   ISA_COND_LT = 2,
   ISA_COND_GE = 3,
   ISA_COND_LE = 4,
   ISA_COND_EQ = 5,
   ISA_COND_NE = 6,
   ISA_COND_AND = 7,
   ISA_COND_OR = 8,
   ISA_COND_XOR = 9,
   ISA_COND_NOT = 10,
   ISA_COND_NZ = 11,
   ISA_COND_GEZ = 12,
   ISA_COND_GZ = 13,
   ISA_COND_LEZ = 14,
   ISA_COND_LZ = 15,
   ISA_COND_SELMSB = 22,
};

enum PACKED isa_swiz {
   ISA_SWIZ_X = 0,
   ISA_SWIZ_Y = 1,
   ISA_SWIZ_Z = 2,
   ISA_SWIZ_W = 3,
};

enum PACKED isa_type {
   ISA_TYPE_F32 = 0,
   ISA_TYPE_S32 = 1,
   ISA_TYPE_S8 = 2,
   ISA_TYPE_U16 = 3,
   ISA_TYPE_F16 = 4,
   ISA_TYPE_S16 = 5,
   ISA_TYPE_U32 = 6,
   ISA_TYPE_U8 = 7,
};

enum PACKED isa_reg_group {
   ISA_REG_GROUP_TEMP = 0,
   ISA_REG_GROUP_INTERNAL = 1,
   ISA_REG_GROUP_UNIFORM_0 = 2,
   ISA_REG_GROUP_UNIFORM_1 = 3,
   ISA_REG_GROUP_TH = 4,
   ISA_REG_GROUP_IMMED = 7,
};

enum PACKED isa_reg_addressing_mode {
   ISA_REG_ADDRESSING_MODE_DIRECT = 0,
   ISA_REG_ADDRESSING_MODE_AX = 1,
   ISA_REG_ADDRESSING_MODE_AY = 2,
   ISA_REG_ADDRESSING_MODE_AZ = 3,
   ISA_REG_ADDRESSING_MODE_AW = 4,
};

enum PACKED isa_rounding {
   ISA_ROUNDING_ = 0,
   ISA_ROUNDING_RTZ = 1,
   ISA_ROUNDING_RTNE = 2,
   ISA_ROUNDING_UNKOWN = 3,
   ISA_ROUNDING_UNKOWN2 = 4,
};

enum PACKED isa_wrmask {
   ISA_WRMASK_____ = 0,
   ISA_WRMASK_X___ = 1,
   ISA_WRMASK__Y__ = 2,
   ISA_WRMASK_XY__ = 3,
   ISA_WRMASK___Z_ = 4,
   ISA_WRMASK_ZX = 5,
   ISA_WRMASK_ZY = 6,
   ISA_WRMASK_XYZ_ = 7,
   ISA_WRMASK____W = 8,
   ISA_WRMASK_X__W = 9,
   ISA_WRMASK__Y_W = 10,
   ISA_WRMASK_XY_W = 11,
   ISA_WRMASK___ZW = 12,
   ISA_WRMASK_X_ZW = 13,
   ISA_WRMASK__YZW = 14,
   ISA_WRMASK_XYZW = 15,
};

enum PACKED isa_thread {
   ISA_THREAD_ = 0,
   ISA_THREAD_T0 = 1,
   ISA_THREAD_T1 = 2,
};

enum PACKED isa_opc {
   ISA_OPC_NOP,
   ISA_OPC_ADD,
   ISA_OPC_MAD,
   ISA_OPC_MUL,
   ISA_OPC_DP3,
   ISA_OPC_DP4,
   ISA_OPC_DSX,
   ISA_OPC_DSY,
   ISA_OPC_MOV,
   ISA_OPC_MOVAR,
   ISA_OPC_RCP,
   ISA_OPC_RSQ,
   ISA_OPC_SELECT,
   ISA_OPC_SET,
   ISA_OPC_EXP,
   ISA_OPC_LOG,
   ISA_OPC_FRC,
   ISA_OPC_CALL,
   ISA_OPC_RET,
   ISA_OPC_BRANCH,
   ISA_OPC_BRANCH_UNARY,
   ISA_OPC_BRANCH_BINARY,
   ISA_OPC_TEXKILL,
   ISA_OPC_TEXLD,
   ISA_OPC_TEXLDB,
   ISA_OPC_TEXLDD,
   ISA_OPC_TEXLDL,
   ISA_OPC_SQRT,
   ISA_OPC_SIN,
   ISA_OPC_COS,
   ISA_OPC_BRANCH_ANY,
   ISA_OPC_FLOOR,
   ISA_OPC_CEIL,
   ISA_OPC_SIGN,
   ISA_OPC_BARRIER,
   ISA_OPC_I2I,
   ISA_OPC_I2F,
   ISA_OPC_F2I,
   ISA_OPC_F2IRND,
   ISA_OPC_CMP,
   ISA_OPC_LOAD,
   ISA_OPC_STORE,
   ISA_OPC_IADDSAT,
   ISA_OPC_IMULLO0,
   ISA_OPC_IMULHI0,
   ISA_OPC_IDIV0,
   ISA_OPC_IMOD,
   ISA_OPC_IMADLO0,
   ISA_OPC_IMADLOSAT0,
   ISA_OPC_MOVAI,
   ISA_OPC_IABS,
   ISA_OPC_LEADZERO,
   ISA_OPC_LSHIFT,
   ISA_OPC_RSHIFT,
   ISA_OPC_ROTATE,
   ISA_OPC_OR,
   ISA_OPC_AND,
   ISA_OPC_XOR,
   ISA_OPC_NOT,
   ISA_OPC_BIT_EXTRACT,
   ISA_OPC_POPCOUNT,
   ISA_OPC_DIV,
   ISA_OPC_ATOMIC_ADD,
   ISA_OPC_ATOMIC_XCHG,
   ISA_OPC_ATOMIC_CMP_XCHG,
   ISA_OPC_ATOMIC_MIN,
   ISA_OPC_ATOMIC_MAX,
   ISA_OPC_ATOMIC_OR,
   ISA_OPC_ATOMIC_AND,
   ISA_OPC_ATOMIC_XOR,
   ISA_OPC_BIT_REV,
   ISA_OPC_TEXLDLPCF,
   ISA_OPC_DP2,
   ISA_OPC_NORM_DP2,
   ISA_OPC_NORM_DP3,
   ISA_OPC_NORM_DP4,
   ISA_OPC_BIT_FINDLSB,
   ISA_OPC_BIT_FINDMSB,
};


#ifdef __cplusplus
} /* extern C */
#endif
